<?php

	require_once('Table.php');
			
	/* ********************************* Données de test ********************************** */
	
	// table de test: liste de produits
	$products = new Table( 
		// en-têtes
		array('productId', 'productLabel', 'productPrice', 'productCategory'),
		// données
		array(
			array(1, 'Ordinateur Asus', 750.00, 1),
			array(2, 'Ordinateur HP', 800.00, 1),
			array(3, 'Téléphone HTC', 250.00, 2),
			array(4, 'Ordinateur Lenovo', 675.00, 1),
			array(5, 'Téléphone iPhone', 5000.00, 2),
			array(6, 'Ordinateur MacOS', 10000.00, 1),
			array(7, 'Ordinateur Acer', 2.00, 1)
		)
	);
	
	// table de test: liste de catégories de produits
	$categories = new Table(
		// en-têtes
		array('categoryId', 'categoryLabel'),
		// données
		array(
			array(1, 'Ordinateur'),
			array(2, 'Téléphone')
		)
	);
?>
<!DOCTYPE html>
<html>
	<head>
		<title>Le fonctionnement des clauses SQL</title>
		<base href="/" />
		<meta charset="UTF-8"/>
		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
		<meta name="viewport" content="width=device-width, maximum-scale=1, user-scalable=1" />
		<style>
			/* Votre mise en forme au besoin */
			html, body{ padding: 0em; margin: 0em; }
			body{ font-family: Arial; background: #dcdcdc; overflow-x: hidden; }
			.content{ width: 70%; background: #fff; margin: auto; }
			h1{ margin: 0px; padding: 1em; text-align: center; }
			h2{ margin: 0px; padding-bottom: 1em; text-align: center; }
			table{ margin: auto; border-collapse: collapse; border: solid 1px #000; }
			table th, table td{ padding: .5em; border: solid 1px #000; border-top: none; border-bottom: dotted 1px #000; }
			table th{ background: rgb(79,129,189); color: #fff; font-weight: normal; text-align: center; }
			table tbody>tr:nth-child(2n)>td{ background: rgb(184,204,228); }
			table td{ text-align: left; background: #fff; }
			.block{ margin: auto; text-align: left; padding: 2em; }
			.block:nth-child(2n){ background: rgb(238,243,248); }
			ul{ display: inline-block; margin: auto; text-align: left; }
			@media (max-width: 1400px){
				.content{ width: 85%; }
			}
			@media (max-width: 1000px){
				.content{ width: 100%; }
			}
		</style>
	</head>
	<body>
		<div class="content">
			
			<h1>TP N°3 - comprendre le fonctionnement des clauses SQL</h1>
			
			<div class="block">
				<h2>Table "Products"</h2>
				Correspond à la requête : <b>SELECT * FROM Products</b> <br/>
				<br/>
				<?=$products->html()?>
			</div>
			
			<div class="block">
				<h2>Table "Categories"</h2>
				Correspond à la requête : <b>SELECT * FROM Categories</b> <br/>
				<br/>
				<?=$categories->html()?>
			</div>
			
			<div class="block">
				<h2>Clause SELECT</h2>
				Correspond à la requête : <b>SELECT productLabel, productPrice FROM Products</b> <br/>
				<br/>
				<?=$products->select(array('productLabel', 'productPrice'))->html()?>
			</div>
			
			<div class="block">
				<h2>Clause WHERE</h2>
				Correspond à la requête : <b>SELECT * FROM Products WHERE productPrice > 900</b> <br/>
				<br/>
				<?=$products->where('productPrice', '>', 900)->html()?>
			</div>
			
			<div class="block">
				<h2>Clauses SELECT + WHERE</h2>
				Correspond à la requête : <b>SELECT productLabel, productPrice FROM Products WHERE productPrice > 900</b> <br/>
				<br/>
				<?=$products->select(array('productLabel', 'productPrice'))->where('productPrice', '>', 900)->html()?>
			</div>
			
			<div class="block">
				<h2>Clause FROM et produit cartésien</h2>
				Correspond à la requête : <b>SELECT * FROM Products, Categories</b> <br/>
				<br/>
				<?=$products->product($categories)->html()?>
			</div>
			
			<div class="block">
				<h2>Jointure classique INNER JOIN</h2>
				Correspond à la requête : <b>SELECT * FROM Products INNER JOIN Cateogies ON productCategory = categoryId</b> <br/>
				<br/>
				<?=$products->innerJoin($categories, 'productCategory', 'categoryId')->html()?>
			</div>
			
			<div class="block">
				<h2>Clause ORDER BY (Plus compliqué)</h2>
				Correspond à la requête : <b>SELECT * FROM Products ORDER BY productPrice DESC</b> <br/>
				<br/>
				<?=$products->orderBy('productPrice DESC')->html()?>
			</div>
			
			<div class="block">
				<h2>Clause GROUP BY (Plus compliqué)</h2>
				Correspond à la requête (fictive): <b>SELECT * FROM Products INNER JOIN Cateogies ON productCategory = categoryId GROUP BY categoryId</b> <br/>
				<br/>
				<i>
					N.B.: on comprend bien qu'il ne s'agisse pas d'une requête "exécutable" dans la mesure où le résultat produit est un tableau à 3 dimensions ! 
					En effet, en utilisant la clause GROUP BY, on a constitué un tableau de "groupes", à savoir un tableau de tableaux à 2 dimensions. Et maintenant 
					que les groupes sont constitués, il faut "agréger" tout cela de sorte qu'à la fin il ne nous reste plus qu'une ligne par groupe.
				</i> <br/>
				<br/>
				<?=$products->innerJoin($categories, 'productCategory', 'categoryId')->groupBy(array('categoryId'))->html()?>
				<br/>
				Correspond à la requête (fictive): <b>SELECT categoryLabel, COUNT(productId) FROM Products INNER JOIN Cateogies ON productCategory = categoryId GROUP BY categoryId</b> <br/>
				<br/>
				<i>N.B.: on comprend bien que, cette fois encore, la requêtes n'est pas "exécutable" puisqu'elle retourne encore une fois un tableau à 3 dimensions !</i> <br/>
				<br/>
				<?=$products->innerJoin($categories, 'productCategory', 'categoryId')->groupBy(array('categoryId'))->select(array('categoryLabel', 'COUNT(productId)'))->html()?>
				<br/>
				Correspond à la requête: <b>SELECT categoryLabel, COUNT(productId) FROM Products INNER JOIN Cateogies ON productCategory = categoryId GROUP BY categoryId, categoryLabel</b> <br/>
				<br/>
				<i>N.B.: on comprend bien que, cette fois-ci, le résultat produit étant bien à 2 dimensions, correspond bien à une requete "exécutable" !</i> <br/>
				<br/>
				<?=$products->innerJoin($categories, 'productCategory', 'categoryId')->groupBy(array('categoryId', 'categoryLabel'))->select(array('categoryLabel', 'COUNT(productId)'))->html()?>
			</div>
			
			<div class="block">
				<h2>Conséquence sur l'ordre d'exécution des clauses</h2>
				Une fois ces algorithmes codés et/ou ces résultats compris, on comprendra sans doute mieux l'ordre d'exécution des clauses SQL. 
				Quoiqu'il en soit, on retiendra : <br/>
				<br/>
				<ul>
					<li>que nécessairement, la clause SELECT (la projection) n'est exécutée qu'à la fin ; </li>
					<li>que les agrégats (COUNT, SUM, AVG, etc.) sont calculés sur la base des regroupements créés à l'issue d'un GROUP BY ; </li>
					<li>qu'une requête contenant une clause GROUP BY ne pourra nécessaiement pas fonctionner si elle retourne un résultat à 3 dimensions ; </li>
					<li>
						que la clause WHERE (selection, i.e. restriction) est toujours exécutée avant le GROUP BY. Dès lors, inutile de chercher à insérer
						des agrégats dans le WHERE, ça ne fonctionnera pas, les regroupements n'étant pas encore effectués ; 
					</li>
					<li>que la clause HAVING (selection, i.e. restriction) n'a qu'une chose de plus qu'une WHERE => elle est executée après exécution d'un GROUP BY. Et c'est bien là son intérêt ; </li>
					<li>qu'en présence d'un GROUP BY, les champs "sélectionnés" (projetés) hors agrégats doivent très normalement figurés dans le GROUP BY (afin d'éviter un résultat invalide à 3 dimensions) ;</li>
					<li>qu'une sous-requête produit complètement similaire à une table. Et c'est bien pour cela qu'on peut utiliser une sous-requête en outre dans la clause FROM.</li>
				</ul>
			</div>
			
			<div class="block">
				<h2>Clause INSERT INTO</h2>
				Correspond à la requête : <b>INSERT INTO Products VALUES (8, "Tablette Microsoft", 500.00, null)</b>, équivalente à la requête 
				<b>INSERT INTO Products(productId, productLabel, productPrice, productCategory) VALUES (8, "Tablette Microsoft", 500.00, null)</b><br/>
				<br/>
				<i>
					N.B.: on comprend bien que, si l'on ne précise pas les champs, l'ordre des valeurs est capital ! En effet, le SGBD ne saurait jouer 
					aux devinettes et ne fera pas d'hypothèse. Les champs de la table ont été créés dans un certain ordre. Et c'est dans cet ordre que 
					le SGBD s'attend à recevoir les valeurs.
				</i> <br/>
				<br/>
				<?=$products->insert(array(8, 'Tablette Microsoft', 500.00, null))->html()?>
			</div>
			
			<div class="block">
				<h2>Clauses DELETE FROM ... WHERE ...</h2>
				Correspond à la requête : <b>DELETE FROM Products WHERE productCategory = 2</b><br/>
				<br/>
				<i>
					N.B.: <br/>
					<ul>
						<li> 
							bien entendu, lors d'un DELETE FROM, le WHERE consiste en une restriction et, en cela, est absolumment similaire au WHERE. Cependant, 
							et vous le savez, il ne s'agit plus de conserver mais de supprimer les lignes pour lesquelles des conditions sont vérifiées ;
						</li>
						<li>
							surtout, on constate qu'il s'agit d'une WHERE habituel, et plus généralement d'une clause comme une autre. Aussi, rien ne nous 
							interdit d'y placer une sous-requête.
						</li>
					</ul>
				</i> <br/>
				<br/>
				<?=$products->delete('productCategory', '=', 2)->html()?>
			</div>
			
			<div class="block">
				<h2>Clauses LEFT et RIGHT JOIN</h2>
				Correspond à la requête : <b>SELECT * FROM Products P LEFT JOIN Categories C ON P.productCategory = C.categoryId</b><br/>
				<br/>
				<i>
					N.B.: on retiendra bien que l'intérêt d'un LEFT JOIN, c'est de pouvoir conserver toutes les lignes de la table de gauche ! 
					Et pas forcément en un seul exemplaire d'ailleurs...
				</i> <br/>
				<br/>
				<?=$products->leftJoin($categories, 'productCategory', 'categoryId')->html()?>
				<br/>
				Strictement équivalent à la requête précédente : <b>SELECT * FROM Categories C RIGHT JOIN Products P ON C.categoryId = P.productCategory</b><br/>
				<br/>
				<i>
					N.B.: RIGHT JOIN, LEFT JOIN... Finalement, c'est la même histoire. Tout LEFT JOIN peut être transformé en RIGHT JOIN et 
					inversement. Le LEFT JOIN conserve les lignes de la table de gauche, le RIGHT JOIN celles de droites...
				</i> <br/>
				<br/>
				<?=$categories->rightJoin($products, 'categoryId', 'productCategory')->html()?>
			</div>
			
			<div class="block">
				<h2>Autre remarques</h2>
				On retiendra également : <br/>
				<br/>
				<ul>
					<li>que le résultat d'une requête SELECT est similaire à une table à ceci près que la requête est calculée !</li>
					<li>
						qu'il est dès lors assez naturel qu'une vue (CREATE/ALTER VIEW) se comporte comme une table. Après tout, une vue
						n'est rien d'autre qu'une requête nommée et stockée, mais recalculée à chaque fois... ; 
					</li>
					<li>
						qu'il est encore assez naturel qu'on puisse utiliser des sous-requêtes dans une clause FROM. Après tout, un FROM 
						attend des tableaux à 2 dimensions. Alors, qu'importe qu'il s'agisse de tables ou de requêtes SELECT.</li>
					</li>
				</ul>
			</div>
			
			<div class="block">
				<h2>Conclusion</h2>
				Tout d'abord, j'espère que ce travail vous a intéressé et a été instructif. J'espère encore que votre travail 
				et l'étude de la correction vous permettra et/ou vous a permis : <br/>
				<br/>
				<ul>
					<li>de vous casser les méninges. Et il est bien naturel que vous ayez rencontré des difficultés ;</li>
					<li>de lever l'ambiguïté sur le fonctionnement des clauses SQL ;</li>
					<li>de comprendre un peu mieux la face cachée des SGBD.</li>
				</ul> <br/>
				<br/>
				Ensuite, je vous suggère de relire ces résultats à tête reposée de sorte que les clauses SQL n'aient plus de secret 
				pour vous !<br/>
				<br/>
				<i>
					Bien entendu, on comprendra qu'il ne s'agit pas d'une version optimisée (performances) et que l'intérêt de ce TP/projet était bel 
					et bien d'en apprendre plus sur le fonctionnement du SQL et sur le fonctionnement interne d'un SGBD.
				</i>
			</div>
			
			<div class="block">
				<big><big>Merci à tous !</big></big>
			</div>
			
		</div>
	</body>
</html>
